#pragma once 

#include "define.h"
#include "SPI.h"
#include "Pin.h"
#include "Point.h"
#include "cmdST7735.h"

class IDisplayDriver
{
protected:
  SPI *spi;
  Pin CS;
  Pin DC;
  Pin RES;
  const uint32_t w;
  const uint32_t h;
  
  void sendCommand ( const uint8_t cmd );
  void sendData    ( const uint8_t data );
  void sendArrData ( uint8_t *data, const uint8_t len );
  void reset       ( );
  
public:
  IDisplayDriver ( SPI *_spi, const Pin &_CS, const Pin &_DC, const Pin &_RES,
                  const uint32_t _w, const uint32_t _h ) 
    : spi(_spi), CS(_CS), DC(_DC), RES(_RES), w(_w), h(_h) {}
    
  virtual void Init() = 0;
  virtual void Clear() = 0;
  virtual void Display( const Point &pt0, const Point &pt1, const uint16_t color ) = 0;
};
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
class ST7789 : public IDisplayDriver
{
public:
  ST7789( SPI *_spi, const Pin &_CS, const Pin &_DC, const Pin &_RES,
                  const uint32_t _w, const uint32_t _h  ) 
    : IDisplayDriver( _spi, _CS, _DC, _RES, _w, _h ) {}
  
  void Init() override;
  void Clear() override;
  void Display( const Point &pt0, const Point &pt1, const uint16_t color ) override; 
};

//------------------------------------------------------------------------------
class ST7735 : public IDisplayDriver
{
public:
  ST7735( SPI *_spi, const Pin &_CS, const Pin &_DC, const Pin &_RES,
                  const uint32_t _w, const uint32_t _h  )
    : IDisplayDriver( _spi, _CS, _DC, _RES, _w, _h ) {}
  
  void Init() override;
  void Clear() override;
  void Display( const Point &pt0, const Point &pt1, const uint16_t color ) override; 
};


