#include "driverEnc28.h"

//------------------------------------------------------------------------------
ENC28J60::ENC28J60( SPI *_spi, Pin *_res, Pin *_cs )
{
  spi = _spi;
  res = _res;
  cs  = _cs;
  idChip = 0;
  currentBunk = 0;
}
//------------------------------------------------------------------------------
void ENC28J60::init( uint8_t *macAddr )
{
  clrRST();
  HAL_Delay(50);
  setRST();
  HAL_Delay(50);
  
//  disChip();
  resetSystem();
  HAL_Delay(10);
  
  writePhyReg( PHLCON, 0x476 );
  
  // configuration of RX buffer 
  writeControlReg16( ERXST, RX_BUF_START );
  writeControlReg16( ERXND, RX_BUF_END );
  writeControlReg16( ERXRDPT, RX_BUF_START);

  writeControlReg( MACON1, MACON1_MARXEN );
  writeControlReg( MACON1, MACON1_TXPAUS );
  writeControlReg( MACON1, MACON1_RXPAUS );
  writeControlReg( MACON3, MACON3_PADCFG0 );
  writeControlReg( MACON3, MACON3_TXCRCEN );
  writeControlReg( MACON3, MACON3_FRMLNEN );
  writeControlReg( MACON3, MACON3_FULDPX );
  
  writeControlReg16( MAMXFL, MAXFRAME );
  writeControlReg( MABBIPG, 0x15 );
  writeControlReg( MAIPG, 0x12 );
  writeControlReg( MAIPG + 1, 0x0C );
  
  writeControlReg( MAADR5, macAddr[0] );
  writeControlReg( MAADR4, macAddr[1] );
  writeControlReg( MAADR3, macAddr[2] );
  writeControlReg( MAADR2, macAddr[3] );
  writeControlReg( MAADR1, macAddr[4] );
  writeControlReg( MAADR0, macAddr[5] );
  
  writePhyReg( PHCON2, PHCON2_HDLDIS );
  writePhyReg( PHCON1, PHCON1_PDPXMD );
  
  bitFieldSet( ECON1, ECON1_RXEN );
  
  idChip = readControlReg( EREVID );
  
  SET_BIT( GPIOD->ODR, GPIO_ODR_OD15 );
}
//------------------------------------------------------------------------------
void ENC28J60::sendPackage( uint16_t len )
{
  SET_BIT( GPIOD->ODR, GPIO_ODR_OD14 ); 
  HAL_Delay( 1 );
//  while ( readControlReg( ECON1 ) & ECON1_TXRTS )
//  {
//    if ( readControlReg( EIR ) & EIR_TXERIF )
//    {
//      bitFieldSet( ECON1, ECON1_TXRTS );
//      bitFieldClear( ECON1, ECON1_TXRTS );
//    }
//  }
  
  writeControlReg16( EWRPT, TX_BUF_START );
  writeControlReg16( ETXST, TX_BUF_START );
  writeControlReg16( ETXND, TX_BUF_START + len );
  writeBufferMem( 0x00, 1 );
  writeBufferMem( txBuffer, len );
  
  
  bitFieldSet( ECON1, ECON1_TXRTS );
  
  idChip = readControlReg( EREVID );
  
  uint16_t etxnd = readControlReg16( ETXND );
  writeControlReg16( ERDPT, etxnd + 1 );
  readBufferMem( txStatusVector, 7 );
}
//------------------------------------------------------------------------------
uint16_t ENC28J60::receivePackage( )
{
  
}
//------------------------------------------------------------------------------
void ENC28J60::setTxBuffer( const void *data, uint16_t len )
{
  const uint8_t *p = static_cast<const uint8_t*>( data );
  
  for ( uint8_t i = 0; i < len; i++ ) 
    txBuffer[i] = p[i];
}
//------------------------------------------------------------------------------
void ENC28J60::getRxBuffer( void *data, uint16_t len )
{
  uint8_t *p = static_cast<uint8_t*>( data );
  
  for ( uint8_t i = 0; i < len; i++ ) 
    p[i] = rxBuffer[i];
}
//------------------------------------------------------------------------------
//---------------------------------Private function-----------------------------
//------------------------------------------------------------------------------
void ENC28J60::checkBank( uint8_t address )
{
  if ( (address & BANK_MASK) != currentBunk )
  {
    write( ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_BSEL1 | ECON1_BSEL0 );
    currentBunk = address & BANK_MASK;
    write( ENC28J60_BIT_FIELD_SET, ECON1, currentBunk >> 5 );
  }
}
//------------------------------------------------------------------------------
void ENC28J60::write( uint8_t opCode, uint8_t address, uint8_t data )
{
  enaChip();
  spi->sendByte( opCode | (address & ADDR_MASK) );
  spi->sendByte( data );
  disChip();
}
//------------------------------------------------------------------------------
uint8_t ENC28J60::read( uint8_t opCode, uint8_t address )
{
  uint8_t data;
  
  enaChip();
  spi->sendByte( opCode | (address & ADDR_MASK) );
  if ( address & SPRD_MASK ) spi->sendByte( 0x00 );
  data = spi->readByte();
  disChip();
  
  return data;
}
//------------------------------------------------------------------------------
uint8_t ENC28J60::readControlReg( uint8_t address )
{
  uint8_t data;
  
  checkBank( address );
  data = read( ENC28J60_READ_CTRL_REG, address );
  
  return data;
}
//------------------------------------------------------------------------------
uint16_t ENC28J60::readControlReg16( uint8_t address )
{
  uint16_t data;
  
  checkBank( address );
  data = read( ENC28J60_READ_CTRL_REG, address ) | 
         (read( ENC28J60_READ_CTRL_REG, address + 1 ) << 8);
  
  return data;
}
//------------------------------------------------------------------------------
void ENC28J60::readBufferMem( uint8_t* data, uint16_t len )
{
  enaChip();
  spi->sendByte( ENC28J60_READ_BUF_MEM );  
  spi->readData( data, len );
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::writeControlReg( uint8_t address, uint8_t data )
{
  checkBank( address );
  write( ENC28J60_WRITE_CTRL_REG, address, data );
}
//------------------------------------------------------------------------------
void ENC28J60::writeControlReg16( uint8_t address, uint16_t data )
{
//  writeControlReg( address, data );
//  writeControlReg( address + 1, data >> 8 );
  checkBank( address );
  write( ENC28J60_WRITE_CTRL_REG, address, data );
  write( ENC28J60_WRITE_CTRL_REG, address + 1, data >> 8);
}
//------------------------------------------------------------------------------
void ENC28J60::writeBufferMem( const void* data, uint16_t len )
{
  enaChip();
  spi->sendByte( ENC28J60_WRITE_BUF_MEM );
  spi->sendData( data, len );
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::bitFieldSet( uint8_t address, uint8_t data )
{
  checkBank( address );
  write( ENC28J60_BIT_FIELD_SET, address, data ); 
}
//------------------------------------------------------------------------------
void ENC28J60::bitFieldClear( uint8_t address, uint8_t data )
{
  checkBank( address );
  write( ENC28J60_BIT_FIELD_CLR, address, data ); 
}
//------------------------------------------------------------------------------
void ENC28J60::resetSystem()
{
  write( ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET );
  currentBunk = 0;
  HAL_Delay(2);
}
//------------------------------------------------------------------------------
uint16_t ENC28J60::readPhyReg( uint8_t address )
{
  writeControlReg( MIREGADR, address );
  writeControlReg( MICMD, MICMD_MIISCAN );
  while ( readControlReg(MISTAT) & MISTAT_BUSY );  
  writeControlReg( MICMD, 0x00 );
  return readControlReg( address ) | (readControlReg( address + 1 ) << 8);
}
//------------------------------------------------------------------------------
void ENC28J60::writePhyReg( uint8_t address, uint16_t data )
{
  writeControlReg( MIREGADR, address );
  writeControlReg( MIWR, data );
  while ( readControlReg(MISTAT) & 0 ); 
}
//------------------------------------------------------------------------------








